#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// 
    /// </summary>
    [Description("")]
    public class TASCDonchian : Indicator
    {
        private int len1 = 40;
        private int len2 = 15;

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.Green), PlotStyle.Line, "LongUpper"));
            Add(new Plot(Color.FromKnownColor(KnownColor.Violet), PlotStyle.Line, "ShortUpper"));
            Add(new Plot(Color.FromKnownColor(KnownColor.Violet), PlotStyle.Line, "ShortLower"));
            Add(new Plot(Color.FromKnownColor(KnownColor.Green), PlotStyle.Line, "LongLower"));
            Overlay				= true;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
            // Use this method for calculating your indicator values. Assign a value to each
            // plot below by replacing 'Close[0]' with your own formula.
            LongUpper.Set(MAX(High, Len1)[0]);
            ShortUpper.Set(MAX(High, Len2)[0]);
            ShortLower.Set(MIN(Low, Len2)[0]);
            LongLower.Set(MIN(Low, Len1)[0]);
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries LongUpper
        {
            get { return Values[0]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries ShortUpper
        {
            get { return Values[1]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries ShortLower
        {
            get { return Values[2]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries LongLower
        {
            get { return Values[3]; }
        }
		
		[Description("")]
        [GridCategory("Parameters")]
        public int Len1
        {
            get { return len1; }
            set { len1 = Math.Max(1, value); }
        }

        [Description("")]
        [GridCategory("Parameters")]
        public int Len2
        {
            get { return len2; }
            set { len2 = Math.Max(1, value); }
        }

        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private TASCDonchian[] cacheTASCDonchian = null;

        private static TASCDonchian checkTASCDonchian = new TASCDonchian();

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public TASCDonchian TASCDonchian(int len1, int len2)
        {
            return TASCDonchian(Input, len1, len2);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public TASCDonchian TASCDonchian(Data.IDataSeries input, int len1, int len2)
        {
            if (cacheTASCDonchian != null)
                for (int idx = 0; idx < cacheTASCDonchian.Length; idx++)
                    if (cacheTASCDonchian[idx].Len1 == len1 && cacheTASCDonchian[idx].Len2 == len2 && cacheTASCDonchian[idx].EqualsInput(input))
                        return cacheTASCDonchian[idx];

            lock (checkTASCDonchian)
            {
                checkTASCDonchian.Len1 = len1;
                len1 = checkTASCDonchian.Len1;
                checkTASCDonchian.Len2 = len2;
                len2 = checkTASCDonchian.Len2;

                if (cacheTASCDonchian != null)
                    for (int idx = 0; idx < cacheTASCDonchian.Length; idx++)
                        if (cacheTASCDonchian[idx].Len1 == len1 && cacheTASCDonchian[idx].Len2 == len2 && cacheTASCDonchian[idx].EqualsInput(input))
                            return cacheTASCDonchian[idx];

                TASCDonchian indicator = new TASCDonchian();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.Len1 = len1;
                indicator.Len2 = len2;
                Indicators.Add(indicator);
                indicator.SetUp();

                TASCDonchian[] tmp = new TASCDonchian[cacheTASCDonchian == null ? 1 : cacheTASCDonchian.Length + 1];
                if (cacheTASCDonchian != null)
                    cacheTASCDonchian.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheTASCDonchian = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TASCDonchian TASCDonchian(int len1, int len2)
        {
            return _indicator.TASCDonchian(Input, len1, len2);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public Indicator.TASCDonchian TASCDonchian(Data.IDataSeries input, int len1, int len2)
        {
            return _indicator.TASCDonchian(input, len1, len2);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TASCDonchian TASCDonchian(int len1, int len2)
        {
            return _indicator.TASCDonchian(Input, len1, len2);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public Indicator.TASCDonchian TASCDonchian(Data.IDataSeries input, int len1, int len2)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.TASCDonchian(input, len1, len2);
        }
    }
}
#endregion
