#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Indicator;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Strategy;
#endregion

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    /// <summary>
    /// Based off of the TASC February 2014 article, The Degree of Complexity, written by Oscar G. Cagigas
    /// </summary>
    [Description("Based off of the TASC February 2014 article, The Degree of Complexity, written by Oscar G. Cagigas")]
    public class DonchianTradingSystem : Strategy
    {
        #region Variables
		
        //User defined inputs
        private int len1 = 40;
        private int len2 = 15;
		private int aTRStop = 4;	
		
		
		//Non UDI
		private double BuyStop;
		private double SellStop;
		private double ShortStop;
		private double CoverStop;
		private double BuyPrice;
		private double ShortPrice;
		private double CoverPrice;
		private double SellPrice;
	
        
        #endregion

        /// <summary>
        /// This method is used to configure the strategy and is called once before any strategy method is called.
        /// </summary>
        protected override void Initialize()
        {
			Add(TASCDonchian(Len1, Len2));
			
			AccountSize = 100000;
			
			EntriesPerDirection = 4;
						
            CalculateOnBarClose = true;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if(CurrentBar < 40)
				return; 
			
			//BuyStop,SellStop
			BuyStop = this.TASCDonchian(Len1, Len2).LongUpper[1];
			SellStop = this.TASCDonchian(Len1, Len2).ShortLower[1];
			ShortStop = this.TASCDonchian(Len1, Len2).LongLower[1];
			CoverStop = this.TASCDonchian(Len1, Len2).ShortUpper[1];
			
			//Exit Orders
			if(Position.MarketPosition == MarketPosition.Flat)
			{
				//Order Entry
				//Long
				if(High[0] > BuyStop) 
				{					
					BuyPrice = Math.Max(BuyStop, Open[0]);
					EnterLongStop(BuyPrice, "Buy Stop");				
				}
				
				//Short
				else if(Low[0] < ShortStop)
				{
					ShortPrice = Math.Min(ShortStop, Open[0]);
					EnterShortStop(ShortPrice, "Short Stop");		
				}
			}
			
			//Sell
			if(Position.MarketPosition == MarketPosition.Long)
			{
				if(Low[0] < SellStop)
				{
					SellPrice = BuyPrice - ATR(ATRStop)[0];
					ExitLongStop(SellPrice, "Buy Stop");
				}
			}
			
			//Cover
			else if(Position.MarketPosition == MarketPosition.Short)
			{
				if(High[0] > CoverStop)
				{
					CoverPrice = ShortPrice + ATR(ATRStop)[0];
					ExitShortStop(CoverPrice, "Short Stop");
				}
			}
			
		}
		
	

        #region Properties
        [Description("")]
        [GridCategory("Parameters")]
        public int Len1
        {
            get { return len1; }
            set { len1 = Math.Max(1, value); }
        }

        [Description("")]
        [GridCategory("Parameters")]
        public int Len2
        {
            get { return len2; }
            set { len2 = Math.Max(1, value); }
        }
		
		[Description("")]
        [GridCategory("Parameters")]
        public int ATRStop
        {
            get { return aTRStop; }
            set { aTRStop = Math.Max(1, value); }
        }
        #endregion
    }
}
